precision highp float;

varying vec2 vTextureCoord;
uniform sampler2D sTexture;

struct AdjustParam {
    float intensity;
    sampler2D tableTexture;
};

uniform AdjustParam brightness;
uniform AdjustParam contrast;
uniform AdjustParam fade;
uniform AdjustParam highlight;
uniform AdjustParam saturation;
uniform AdjustParam shadow;
uniform AdjustParam temperature;
uniform AdjustParam tone;

uniform float quadCountPerRow;
uniform float tableWidth;
uniform float tableHeight;

vec4 handleWithTable(vec4 curColor, sampler2D tableTexture, int tableRows, bool preMultiALpha, float intensity) {
    float quadIndex = curColor.b * (quadCountPerRow - 1.0);
    float xInQuad = (1.0 / quadCountPerRow - 1.0 / tableWidth) * curColor.r;
    float yInQuad = (1.0 - 1.0 / tableHeight) * curColor.g;

    vec2 quadFloor = vec2(floor(quadIndex), 0.0);
    vec2 quadCeil = vec2(ceil(quadIndex), 0.0);
    vec2 textureCoordFloor = vec2(quadFloor.x / quadCountPerRow + xInQuad + 0.5 / tableWidth, quadFloor.y + yInQuad + 0.5 / tableHeight);
    vec2 textureCoordCeil = vec2(quadCeil.x / quadCountPerRow + xInQuad + 0.5 / tableWidth, quadCeil.y + yInQuad + 0.5 / tableHeight);

    if (tableRows == 2) {
        float offset = step(0.0, intensity) * 0.5;
        textureCoordFloor.y = textureCoordFloor.y * 0.5 + offset;
        textureCoordCeil.y = textureCoordCeil.y * 0.5 + offset;
    }

    vec4 newColorFloor = texture2D(tableTexture, textureCoordFloor);
    vec4 newColorCeil = texture2D(tableTexture, textureCoordCeil);
    vec4 newColor = mix(newColorFloor, newColorCeil, fract(quadIndex));

    newColor = mix(curColor, newColor, abs(intensity));
    newColor.a = curColor.a;
    if (preMultiALpha) {
        newColor.rgb *= curColor.a;
    }
    return newColor;
}

vec4 doBrightness(vec4 curColor) {
    if (brightness.intensity != 0.0) {
        curColor = handleWithTable(curColor, brightness.tableTexture, 2, false, brightness.intensity);
    }
    return curColor;
}

vec4 doContrast(vec4 curColor) {
    if (contrast.intensity != 0.0) {
        curColor = handleWithTable(curColor, contrast.tableTexture, 2, false, contrast.intensity);
    }
    return curColor;
}

vec4 doSaturation(vec4 curColor) {
    if (saturation.intensity != 0.0) {
        curColor = handleWithTable(curColor, saturation.tableTexture, 2, false, saturation.intensity);
    }
    return curColor;
}

vec4 doHighlight(vec4 curColor) {
    if (highlight.intensity != 0.0) {
        curColor = handleWithTable(curColor, highlight.tableTexture, 1, false, highlight.intensity);
    }

    return curColor;
}

vec4 doShadow(vec4 curColor) {
    if (shadow.intensity != 0.0) {
        curColor = handleWithTable(curColor, shadow.tableTexture, 1, true, shadow.intensity);
    }
    return curColor;
}

vec4 doTemperature(vec4 curColor) {
    if (temperature.intensity != 0.0) {
        curColor = handleWithTable(curColor, temperature.tableTexture, 2, false, temperature.intensity);
    }
    return curColor;
}

vec4 doTone(vec4 curColor) {
    if (tone.intensity != 0.0) {
        curColor = handleWithTable(curColor, tone.tableTexture, 2, false, tone.intensity);
    }
    return curColor;
}

vec4 doFade(vec4 curColor) {
    if (fade.intensity != 0.0) {
        curColor = handleWithTable(curColor, fade.tableTexture, 1, true, fade.intensity);
    }
    return curColor;
}

void main()
{
    vec4 curColor = texture2D(sTexture, vTextureCoord);

    curColor = doBrightness(curColor);
    curColor = doContrast(curColor);
    curColor = doSaturation(curColor);
    curColor = doHighlight(curColor);
    curColor = doShadow(curColor);
    curColor = doTemperature(curColor);
    curColor = doTone(curColor);
    curColor = doFade(curColor);

    gl_FragColor = curColor;
}
